<?php

/*************************************************************************
 * Username Change - XenConcept (c) 2017
 * All Rights Reserved.
 **************************************************************************
 * This file is subject to the terms and conditions defined in the Licence
 * Agreement available at Try it like it buy it :)
 *************************************************************************/

namespace XenConcept\UsernameChange\XF\Entity;

use XF\Mvc\Entity\Structure;


/**
 * COLUMNS
 * @property int last_username_change
 *
 * GETTERS
 * @property string has_username_change_pending
 */
class User extends XFCP_User
{

    // ############## GET METHOD ##############

    public function getDaysBetweenChange($forDisplay = false)
    {
        $daysBetweenChange = $this->getCustomPermissions('xc_change_username', 'daysBetweenEachChange');

        if ($forDisplay)
        {
            if ($daysBetweenChange == -1 || $daysBetweenChange == 0)
            {
                return \XF::phrase('unlimited');
            }
            else
            {
                return \XF::phrase('xc_username_change_x_days', ['days' => $daysBetweenChange]);
            }
        }
        else
        {
            return $daysBetweenChange;
        }
    }

    public function getNumberOfPostsToChangeTheUsername()
    {
        return $this->getCustomPermissions('xc_change_username', 'minPostCountForChange');
    }

    // ############## PERM METHOD ##############

    public function canUsernameChange($includeOtherRestrict = false, &$error = null)
    {
        if (!$this->hasPermission('xc_change_username', 'change'))
        {
            return false;
        }

        if ($includeOtherRestrict)
        {
            $messageRequired = $this->getNumberOfPostsToChangeTheUsername();

            if ($messageRequired > $this->message_count)
            {
                $error = \XF::phraseDeferred('xc_username_change_you_must_have_x_posts_to_be_able_to_change_of_username_currently_you_have_x_posts', [
                    'messageRequired' => $messageRequired,
                    'currentMessage'  => $this->message_count
                ]);

                return false;
            }

            if ($this->last_username_change)
            {
                $daysBetweenChange = $this->getDaysBetweenChange();

                if ($daysBetweenChange != -1 || $daysBetweenChange != 0)
                {
                    if (\XF::$time < $this->last_username_change + 86400 * $daysBetweenChange)
                    {
                        $changeDay = $this->last_username_change + 86400 * $daysBetweenChange;
                        $changeDay = \XF::language()->date($changeDay);

                        $error = \XF::phraseDeferred('xc_username_change_you_can_change_of_username_again_the_x', ['changeDay' => $changeDay]);
                        return false;
                    }
                }
            }
        }

        return true;
    }

    public function canUseUsernameChangePrivate()
    {
        return $this->hasPermission('xc_change_username', 'useChangePrivate');
    }

    public function canBypassApprovalChange(&$error = null)
    {
        $visitor = \XF::visitor();

        if (!$visitor->user_id)
        {
            return false;
        }

        return $visitor->hasPermission('xc_change_username', 'bypassApprovalOfChange');
    }

    public function canViewLogsUsername(&$error = null)
    {
        $visitor = \XF::visitor();

        if (!$visitor->user_id)
        {
            return false;
        }

        if ($visitor->user_id == $this->user_id)
        {
            if (!$visitor->hasPermission('xc_change_username', 'viewOwnLogs'))
            {
                return false;
            }
        }
        else
        {
            if (!$visitor->hasPermission('xc_change_username', 'viewAnyLogs'))
            {
                return false;
            }
        }

        if ($visitor->user_id != $this->user_id)
        {
            if ($visitor->canBypassUserPrivacy())
            {
                return true;
            }

            if (!$this->isPrivacyCheckMet('allow_view_history_username_change', $visitor))
            {
                $error = \XF::phraseDeferred('xc_username_change_the_user_does_not_want_to_show_the_history_of_changes_to_his_username');
                return false;
            }
        }

        return true;
    }

    public function canClearAnyLogsUsername(&$error = null)
    {
        $visitor = \XF::visitor();

        if (!$visitor->user_id)
        {
            return false;
        }

        return $visitor->hasPermission('xc_change_username', 'clearAnyLogs');
    }

    public function canClearOwnLogsUsername(&$error = null)
    {
        $visitor = \XF::visitor();

        if (!$visitor->user_id)
        {
            return false;
        }

        if ($visitor->user_id != $this->user_id)
        {
            return false;
        }

        return $visitor->hasPermission('xc_change_username', 'clearOwnLogs');
    }

    public function canClearLogsUsername(&$error = null)
    {
        $visitor = \XF::visitor();

        if (!$visitor->user_id)
        {
            return false;
        }

        if ($visitor->user_id != $this->user_id)
        {
            return $visitor->hasPermission('xc_change_username', 'clearAnyLogs');
        }

        return $visitor->hasPermission('xc_change_username', 'clearOwnLogs');
    }

    public function canApproveChangeUsername(&$error = null)
    {
        $visitor = \XF::visitor();

        if (!$visitor->user_id)
        {
            return false;
        }

        return $visitor->hasPermission('xc_change_username', 'approveUsername');
    }


    // ############## STRUCTURE FUNCTION ##############


    public function getHasUsernameChangePending()
    {
        $lastChange = $this->getChangeLogRepo()->findLastChangeLog($this->user_id);

        if ($lastChange && $lastChange->change_state == 'moderated')
        {
            return true;
        }

        return false;
    }


    public static function getStructure(Structure $structure)
    {
        $structure = parent::getStructure($structure);

        $structure->columns['last_username_change'] = ['type' => self::UINT, 'default' => 0];

        $structure->getters['has_username_change_pending'] = true;

        return $structure;
    }


    // ############## GET CUSTOM PERMISSION ##############


    public function getCustomPermissions($group, $permission)
    {
        $userGroupsIds   = $this->secondary_group_ids;
        $userGroupsIds[] = $this->user_group_id;

        $finder = $this->finder('XF:PermissionEntry');
        $finder
            ->where('permission_group_id', $group)
            ->where('permission_id', $permission)
            ->where('user_group_id', $userGroupsIds)
            ->setDefaultOrder('permission_value_int');

        $permissionEntries = $finder->fetchRaw();

        $value = 0;

        /**
         * @var \XF\Entity\PermissionEntry $permissionEntry
         */
        foreach ($permissionEntries AS $permissionEntry)
        {
            $permissionValueInt = $permissionEntry['permission_value_int'];

            if ($permissionValueInt == '-1')
            {
                $value = '-1';
                break;
            }
            else if ($permissionValueInt > 0)
            {
                $value = $permissionValueInt;
                break;
            }
        }

        return $value;
    }

    // ############## GET REPO ##############

    /**
     * @return \XenConcept\UsernameChange\Repository\ChangeLog
     */
    protected function getChangeLogRepo()
    {
        return $this->repository('XenConcept\UsernameChange:ChangeLog');
    }

}